#ifndef __CAMERA_H__
#define __CAMERA_H__

#include "geommath.h"

class CCamera
{
public:
	void SetClipPlanes(float fFrontClip, float fBackClip);	// Should both be negative
	void SetPosViewPlane(const Vec3 &vVRP);
	void SetRotViewPlane(const Vec3 &vVPN, const Vec3 &vVUP);
	void SetBoundVRC(float fUmin, float fVmin, float fUmax, float fVmax);
	void SetEyePos(const Vec3 &vPRP);

	void SetBitmapSize(float fPixelWidth, float fPixelHeight, float fZBufDepth);
	void SetViewPortOffset(float fViewOffsX, float fViewOffsY, float fViewOffsZ);

	CCamera();

public:
	const Mat44 GetWorldToBitmapTransform();	// Alle Matrices put together
	const Mat44 GetTransformCamToWorld();
	const Mat44 GetPerspectiveTransform();
	const Mat44 GetViewportTransform();

private:
	void SetupMatrices();

	Mat44 m_m44WorldToBitmapTransform;
	Mat44 m_m44TransformCamToWorld;
	Mat44 m_m44PerspectiveTransform;
	Mat44 m_m44ViewportTransform;

	float m_fPixelWidth;
	float m_fPixelHeight;
	float m_fZBufDepth;

	float m_fViewPortOffsX;		// Used to offset the screen-coordinates
	float m_fViewPortOffsY;
	float m_fViewPortOffsZ;

private:
	Vec3 m_vPRP;
	float m_fUmin, m_fVmin;
	float m_fUmax, m_fVmax;

	Vec3 m_vVRP, m_vVPN, m_vVUP;

	float m_fFrontClip, m_fBackClip;
};



#endif
