#include "dmawrap.h"

sps2Memory_t * CDmaWrap::m_pDescCached=NULL;
sps2Memory_t * CDmaWrap::m_pDescUncached=NULL;


void CDmaWrap::DisableUCAB()
{
	assert(!m_bPacketInDevelopment);
	assert(!m_bCacheEnabled);
	assert(!m_bTransferFromScratchPad);
	//assert(((uint32)m_pDmaMem&DMAFROMSP)==0);

	uint32 uOffsDmaMemToCur = ((uint32) m_pCurLocation) - ((uint32) m_pDmaMem);
	uint32 uOffsBaseToDmaMem = ((uint32) m_pDmaMem) - ((uint32) m_pDescUncached->pvStart);
	m_pDmaMem = (void *) (uOffsBaseToDmaMem + ((uint32) m_pDescCached->pvStart));
	m_pCurLocation = (void *) (uOffsDmaMemToCur + ((uint32) m_pDmaMem));
	m_bCacheEnabled = true;
}

void CDmaWrap::EnableUCAB()
{
	assert(!m_bPacketInDevelopment);
	assert(m_bCacheEnabled);
	assert(!m_bTransferFromScratchPad);
	//assert(((uint32)m_pDmaMem&DMAFROMSP)==0);

	uint32 uOffsDmaMemToCur = ((uint32) m_pCurLocation) - ((uint32) m_pDmaMem);
	uint32 uOffsBaseToDmaMem = ((uint32) m_pDmaMem) - ((uint32) m_pDescCached->pvStart);
	m_pDmaMem = (void *) (uOffsBaseToDmaMem + ((uint32) m_pDescUncached->pvStart));
	m_pCurLocation = (void *) (uOffsDmaMemToCur + ((uint32) m_pDmaMem));
	m_bCacheEnabled = false;
}






CDmaWrap::CDmaWrap(void * pDmaMem, uint32 uDmaQuadSize, TRANS_MODE trans_mode)
{
	// Check for transfer from ScratchPad
	// check if the distance is less then 16kb
	m_bTransferFromScratchPad = ((uint32) pDmaMem) - ((uint32) SCRATCH_PAD) < 0x4000;

	m_bCacheEnabled = true;
	m_pDmaMem = ForceToCachedVirtPtr(pDmaMem);
	m_pCurLocation = m_pDmaMem;
	m_uDmaQuadSize = uDmaQuadSize;

	assert((uint32(m_pDmaMem)&0xf)==0);	// must be quad-aligned
		
	m_bSetTTE = false;
	m_uTransMode = trans_mode;

	// Init Packet Logic Vars
	m_bPacketInDevelopment = false;
	m_pRecordLocation = NULL;

		//if(((uint32)m_pDmaMem&DMAFROMSP)==0)
	if(!m_bTransferFromScratchPad)
		EnableUCAB();	// Enabled by default

	// PS2Linux specific details
	m_pStartPhys = VirtToPhys(pDmaMem);
}

CDmaWrap::~CDmaWrap()
{
}


// PS2Linux specific details
void * CDmaWrap::VirtToPhys(void * pVirtPtr)
{
	// check for ScratchPad mem
	if(((uint32) pVirtPtr) - ((uint32) SCRATCH_PAD) < 0x4000)
		return (void *) (0xf0000000L+(((uint32) pVirtPtr)-((uint32) SCRATCH_PAD)));
	else if(m_pDescCached->pvStart<=pVirtPtr && pVirtPtr<m_pDescCached->pvEnd)
		return (void *) sps2GetPhysicalAddress(pVirtPtr, m_pDescCached);
	else
	{
		assert(m_pDescUncached->pvStart<=pVirtPtr && pVirtPtr<m_pDescUncached->pvEnd);
		return (void *) sps2GetPhysicalAddress(pVirtPtr, m_pDescUncached);
	}
}

void * CDmaWrap::ForceToInternalPtr(void * pPtrIn)
{
	if(m_bCacheEnabled)
		return ForceToCachedVirtPtr(pPtrIn);
	else
		return ForceToUncachedVirtPtr(pPtrIn);	
}

void * CDmaWrap::ForceToCachedVirtPtr(void * pPtrIn)
{
	void * pPtrOut = pPtrIn;
	
	// check if we're in scratchpad area
	if(((uint32) pPtrIn) - ((uint32) SCRATCH_PAD) >= 0x4000)
	{
		if(m_pDescUncached->pvStart<=pPtrIn && pPtrIn<m_pDescUncached->pvEnd)
		{
			uint32 uOffs = ((uint32) pPtrIn) - ((uint32) m_pDescUncached->pvStart);
			pPtrOut = (void *) (((uint32) m_pDescCached->pvStart) + uOffs);
		}

		assert(m_pDescCached->pvStart<=pPtrOut && pPtrOut<m_pDescCached->pvEnd);
	}

	return pPtrOut;
}

void * CDmaWrap::ForceToUncachedVirtPtr(void * pPtrIn)
{
	void * pPtrOut = pPtrIn;
	
	// check if we're in scratchpad area
	if(((uint32) pPtrIn) - ((uint32) SCRATCH_PAD) >= 0x4000)
	{
		if(m_pDescCached->pvStart<=pPtrIn && pPtrIn<m_pDescCached->pvEnd)
		{
			uint32 uOffs = ((uint32) pPtrIn) - ((uint32) m_pDescCached->pvStart);
			pPtrOut = (void *) (((uint32) m_pDescUncached->pvStart) + uOffs);
		}

		assert(m_pDescUncached->pvStart<=pPtrOut && pPtrOut<m_pDescUncached->pvEnd);
	}

	return pPtrOut;
}
