#include "imeshinstance.h"
#include "macro_codes.h"
#include "imesh.h"
#include <math.h>
#include "p2lassert.h"
#include "stitchhack.h"


void IMeshInstance::Draw(Mat44 m, Mat44 ToScr)
{
	for(int a=0; a<8; a++)
	{
		memcpy(m_pLocToWorld[a], (void *) &m.m_fMat[(a>>1)*4+((a&1)<<1)], 8);
		memcpy(m_pWorldToScreen[a], (void *) &ToScr.m_fMat[(a>>1)*4+((a&1)<<1)], 8);
	}
	asm("sync.l");
}

void IMeshInstance::InitDMA_List()
{
	InitChain(m_pDmaVif1, STCYCL(1,1));
	AddWordToChain(m_pDmaVif1, UNPACK_TO_TOP(V4_32, 8, PAD_CHUNK+0));	

	// clear space for, LocalToWorld and WorldToScreen
	for(int a=0; a<4; a++)
	{
		// beware of TTE/NewTag discontinuence
		m_pLocToWorld[a*2+0] = AddWordToChain(m_pDmaVif1, 0);
		AddWordToChain(m_pDmaVif1, 0);
		m_pLocToWorld[a*2+1] = AddWordToChain(m_pDmaVif1, 0);
		AddWordToChain(m_pDmaVif1, 0);
	}

	for(int a=0; a<4; a++)
	{
		// beware of TTE/NewTag discontinuence
		m_pWorldToScreen[a*2+0] = AddWordToChain(m_pDmaVif1, 0);
		AddWordToChain(m_pDmaVif1, 0);
		m_pWorldToScreen[a*2+1] = AddWordToChain(m_pDmaVif1, 0);
		AddWordToChain(m_pDmaVif1, 0);
	}

	// setup info on obj
	AddWordToChain(m_pDmaVif1, UNPACK_TO_TOP(V4_32, 1, PAD_CHUNK+19));
	AddWordToChain(m_pDmaVif1, 0);	// 0 clipping planes
	float fVar = 0;
	AddWordToChain(m_pDmaVif1, *((uint32 *) &fVar));	// don't remember this one
	fVar = -255.0f / (1000.0f - 20.0f);
	AddWordToChain(m_pDmaVif1, *((uint32 *) &fVar));	// fogval1
	fVar = -1000.0f * fVar;
	AddWordToChain(m_pDmaVif1, *((uint32 *) &fVar));	// fogval2


	AddWordToChain(m_pDmaVif1, UNPACK_TO_TOP(V4_32, 3, PAD_CHUNK+20));

	static bool bOddRun = false;
	uint32 uAddAddr = (bOddRun == false) ? 0 : 8; 

	uint32 uDstPtr = ((496+uAddAddr)*2048)/64;
	uint64 uVal1 = GIFTAG(2, 1, 0, 0, 0, 1);
	uint64 uVal2 = GIFTAG_REGS1(AD);
	
	uint64 uVal3 = 0;
	uint64 uVal4 = TEXFLUSH;
	uint64 uVal5 = TEX0_SET(uDstPtr, 2, 0, 7, 7, 1, 0, 0, 0, 0, 0, 0);
	uint64 uVal6 = TEX0_1;

	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal1)[0]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal1)[1]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal2)[0]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal2)[1]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal3)[0]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal3)[1]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal4)[0]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal4)[1]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal5)[0]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal5)[1]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal6)[0]);
	AddWordToChain(m_pDmaVif1, ((uint32 *) &uVal6)[1]);

	

	bOddRun = (bOddRun == false) ? true : false;
	
	// Terminate (add a ret_tag)
	Terminate(m_pDmaVif1);	
}



                                      
IMeshInstance::IMeshInstance(IMesh * pMotherMesh, CDmaVif1 * pDmaVif1)
{
	m_pMotherMesh = pMotherMesh;
	m_pDmaVif1 = (CDmaVif1 *) malloc(sizeof(CDmaVif1));
	*m_pDmaVif1 = *pDmaVif1;
	m_pDmaVif1->Reset();
	m_pDmaTags = m_pDmaVif1->GetCurLocation();

	InitDMA_List();

	m_iNumClipPlanes = 0;
}

