#include <stdio.h>
#include <assert.h>
#include "exttypes.h"
#include "sps2lib.h"

int iSPS2Device;
sps2Memory_t * pDescriptor;
sps2Memory_t * pDescriptor2;

void InitMatDma(uint32 * pTblSrc1, uint32 * pTblSrc2, uint32 * pTblDst, uint64 * pChainMem, uint64 * pPhysChain);
void MatMulMat(void * pSrc1, void * pSrc2, void * pDst, uint32 uTotNumMats);

void * magicalloc(uint32 uRequestedBytes)
{
	pDescriptor = sps2Allocate(uRequestedBytes, SPS2_MAP_BLOCK_4K, iSPS2Device);

	if(pDescriptor)
		return pDescriptor->pvStart;

	printf("allocation failed!\n");
	assert(false);
}

void * UncachedBlockAlloc()
{
	pDescriptor2 = sps2Allocate(4096, SPS2_MAP_BLOCK_4K | SPS2_MAP_UNCACHED, iSPS2Device);

	if(pDescriptor2)
		return pDescriptor2->pvStart;

	printf("Could create Chain!\n");
	assert(false);
}

int main()
{
	// Open SPS2Device
	iSPS2Device = sps2Init();

	if (iSPS2Device<0) {
	  printf("Can't open SPS2 device.  Is the module loaded?\n");
	  exit(-1);
	}

	int iVPU0=open("/dev/ps2vpu0",O_RDWR,0);

	// setup interlock
	const uint32 uAllocMegs = 6;
	void * pSrc1 = magicalloc(uAllocMegs*(1024*1024));

	const uint32 uArrayLen = 1024*1024*(uAllocMegs/3);
	void * pSrc2 = ((char *) pSrc1) + uArrayLen;
	void * pDst = ((char *) pSrc2) + uArrayLen;

	// Lookup-tables
	const uint32 uTabLen = uArrayLen / 4096;
	uint32 uTblSrc1[uTabLen];
	uint32 uTblSrc2[uTabLen];
	uint32 uTblDst[uTabLen];

	// Fill up matrices with something
	//memset(pSrc1, uAllocMegs*(1024*1024), 0);
	for(uint32 k=0; k<(2*(1024*1024))/48; k++)
	{
		for(int i=0; i<2; i++)
		{
			uint32 uAddr = (i==0) ? (((uint32) pSrc1) + k*48) : (((uint32) pSrc2) + k*48);
			((float *) uAddr)[0] = 1;
			((float *) uAddr)[1] = 0;
			((float *) uAddr)[2] = 0;
			((float *) uAddr)[3] = 0;
			((float *) uAddr)[4] = 1;
			((float *) uAddr)[5] = 0;
			((float *) uAddr)[6] = 0;
			((float *) uAddr)[7] = 0;
			((float *) uAddr)[8] = 1;
			((float *) uAddr)[9] = 9;
			((float *) uAddr)[10] = 10;
			((float *) uAddr)[11] = 11;
		}
	}


	for(int i=0; i<uTabLen; i++)
	{
		uTblSrc1[i] = sps2GetPhysicalAddress((void *) (((uint32) pSrc1) + i*4096), pDescriptor);
		uTblSrc2[i] = sps2GetPhysicalAddress((void *) (((uint32) pSrc2) + i*4096), pDescriptor);
		uTblDst[i] = sps2GetPhysicalAddress((void *) (((uint32) pDst) + i*4096), pDescriptor);
	}

	// Under the module this needs to be allocated using the module
	//uint64 uChainMem[10*2] __attribute__((aligned(64)));
	void * pChain = 
(uint64 *) UncachedBlockAlloc();

	uint32 uNumMats = (uArrayLen / 48);
	sps2FlushCache(iSPS2Device);

	// This is the part that ordinarily would run on a per frame basis, the rest is done at initialization time (first frame)
	InitMatDma(uTblSrc1, uTblSrc2, uTblDst, (uint64 *) pChain, (uint64 *) sps2GetPhysicalAddress(pChain, pDescriptor2));
	MatMulMat(pSrc1, pSrc2, pDst, uNumMats);
	// End

	// print out some result (Just the first mat)
	printf("Success!!\n");
	uint32 uLoc = 0;
	printf("%f %f %f %f\n", ((float *) pDst)[uLoc+0], ((float *) pDst)[uLoc+1], ((float *) pDst)[uLoc+2], ((float *) pDst)[uLoc+3]);
	printf("%f %f %f %f\n", ((float *) pDst)[uLoc+0+4], ((float *) pDst)[uLoc+1+4], ((float *) pDst)[uLoc+2+4], ((float *) pDst)[uLoc+3+4]);
	printf("%f %f %f %f\n", ((float *) pDst)[uLoc+0+8], ((float *) pDst)[uLoc+1+8], ((float *) pDst)[uLoc+2+8], ((float *) pDst)[uLoc+3+8]);

	//delete [] pAlloc;
}
