#undef DEBUG_DMA

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include "dmavif1.h"
#include "screen.h" 
#include "geommath.h"
#include "camera.h"
#include "p2lassert.h"

int loop_again=1;

// this is executed when ctrl-c is pressed, ideally it stops the loop.
// the deinit functions have to be at the end of the main after the loop.
void sig_handle(int sig)
{
	loop_again=0;
}


// distribute sps2 specific global pointers to relevant classes
void  DistribGlobPtrs(void * pDescCached, void * pDescUncached)
{
	CDmaWrap::m_pDescCached = (sps2Memory_t *) pDescCached;
	CDmaWrap::m_pDescUncached = (sps2Memory_t *) pDescUncached;
}

// labels for vu1 dummy-code
extern "C" void VU_vu1dummy_start(void);
extern "C" void VU_vu1dummy_end(void);

int iSPS2Device;
CScreen *cScreen=NULL;

void delete_screen()
{
	if (cScreen!=NULL) delete cScreen;
	cScreen=NULL;
}

int main()
{
	// Open sps2-library and setup PS2Linux stuff
	iSPS2Device = sps2Init();

	if (iSPS2Device<0) {
	  printf("Can't open SPS2 device.  Is the module loaded?\n");
	  exit(-1);
	}

	uint32 uAllocDmaMegs = 2;
	sps2Memory_t * pDescCached = sps2Allocate(uAllocDmaMegs*1024*1024, SPS2_MAP_BLOCK_4K | SPS2_MAP_CACHED, iSPS2Device);
	sps2Memory_t * pDescUncached = sps2Remap(pDescCached, SPS2_MAP_UNCACHED, iSPS2Device);
	DistribGlobPtrs(pDescCached, pDescUncached);
	void * pAllDmaMem = pDescCached->pvStart;
	int iVPU0; iVPU0=open("/dev/ps2vpu0", O_RDWR);
	int iVPU1; iVPU1=open("/dev/ps2vpu1", O_RDWR);
	for(int sig=0; sig<128; sig++)	// catch as many signals as possible
		signal(sig, sig_handle);

	
	// Setup Screen
	cScreen=new CScreen;
	atexit(delete_screen);
	cScreen->ScreenSetup(pAllDmaMem, 16);

	// Setup a Camera
	float fScrWdth = cScreen->GetWidth();
	float fScrHgt = cScreen->GetHeight();
	float fDepthZ = cScreen->GetDepth();
	CCamera cCam;
	cCam.SetBitmapSize(fScrWdth, -fScrHgt, fDepthZ);
	cCam.SetViewPortOffset(2048, 2048, fDepthZ);
	cCam.SetClipPlanes(-10, -10000);
	cCam.SetEyePos(Vec3(0,0,1));
	float fFov = 0.524f;	// total view of 60 degrees
	float fMinU = -tanf(fFov);
	float fMinV = (fMinU*3)/4;
	cCam.SetBoundVRC(fMinU, fMinV, -fMinU, -fMinV);
	cCam.SetPosViewPlane(Vec3(0,0,600));
	cCam.SetRotViewPlane(Vec3(0,0,1), Vec3(0,1,0));
	

	// Flush Cache
	sps2FlushCache(iSPS2Device);

	while(loop_again)
	{
		// tmp
		static uint32 uVal = 0;
		CDmaVif1 dmaVif1(((char *) pAllDmaMem)+16*16, 11);
		dmaVif1.AttachSrcEndTag(DIRECT(10)<<32);
		uint32 nloop = 9;
		uint32 uPrim = PRIM_SET(3,1,0,0,0,0,0,0,0);
		dmaVif1.add64(GIFTAG(nloop, 1, 1, uPrim, 0, 1));
		dmaVif1.add64(GIFTAG_REGS1(AD));

		dmaVif1.add64(ST_SET(0,0));
		dmaVif1.add64(ST);
		dmaVif1.add64(RGBAQ_SET(127,0,0,0,0x3f800000));
		dmaVif1.add64(RGBAQ);
		dmaVif1.add64(XYZ2_SET((-126+2048+uVal)*16,(-60+2048)*16, 500));
		dmaVif1.add64(XYZ2);

		dmaVif1.add64(ST_SET(0,0x3f800000));
		dmaVif1.add64(ST);
		dmaVif1.add64(RGBAQ_SET(0,127,0,0,0x3f800000));
		dmaVif1.add64(RGBAQ);
		dmaVif1.add64(XYZ2_SET((160+2048+uVal)*16,(-50+2048)*16, 500));
		dmaVif1.add64(XYZ2);
		
		dmaVif1.add64(ST_SET(0x3f800000,0));
		dmaVif1.add64(ST);
		dmaVif1.add64(RGBAQ_SET(0,0,127,0,0x3f800000));
		dmaVif1.add64(RGBAQ);
		dmaVif1.add64(XYZ2_SET((-150+2048)*16,(120+2048)*16, 500));
		dmaVif1.add64(XYZ2);
		uVal = (uVal+1)%20;
		dmaVif1.EndPacket();
		dmaVif1.Launch(iSPS2Device);
		dmaVif1.Wait();

		cScreen->Swap();
	}

	return 0;
}
