#include "macro_codes.h"
#include <string.h>
#include <math.h>
#include "dmagif.h"


void GenText1(uint8 * pPic);
void GenText2(uint8 * pPic);
void GenText3(uint8 * pPic);
void GenText4(uint8 * pPic);

void AddTexToChain(CDmaGif * pDmaGif, uint32 uVRamPage, void * pImgData);
void SyncStitch(CDmaWrap * pDmaWrap);


CDmaGif GenerateTextures(void * pDmaMem, uint32 uMemBytesLeft)
{
	// Build textures
	assert(uMemBytesLeft > 64*1024);
	GenText2((uint8 *) pDmaMem);

	// Build chain
	CDmaGif dmaGif((void *) (((uint8 *) pDmaMem)+64*1024), (uMemBytesLeft-64*1024)>>4);
	AddTexToChain(&dmaGif, 496, (void *) pDmaMem);

	// close up
	SyncStitch(&dmaGif);
	dmaGif.AttachSrcEndTag();
	dmaGif.EndPacket();

	return dmaGif;
}

void GenText1(uint8 * pPic)
{
	for(int iY=0; iY<128; iY++)
		for(int iX=0; iX<128; iX++)
		{
			uint32 uOffs = (iY*128+iX)*4;

			float fR = 255*tanf(((float) iX)/127);
			float fG = 0;
			float fB = 63;

			pPic[uOffs+0] = (uint8) fR;
			pPic[uOffs+1] = (uint8) fG;
			pPic[uOffs+2] = (uint8) fB;
			pPic[uOffs+3] = 0x7f;
		}
}

void GenText2(uint8 * pPic)
{
	for(int iY=0; iY<128; iY++)
		for(int iX=0; iX<128; iX++)
		{
			uint32 uOffs = (iY*128+iX)*4;
			const float PI = (3.141592f);

			float fR = 255*cosf(((float) iX)/127);
			float fG = (((iX>>3)&1) ^ ((iY>>3)&1))*127;
			float fB = 255*sinf(((float) iX*PI*0.5f)/127);
			if(fB<0) fB=0;

			pPic[uOffs+0] = (uint8) fR;
			pPic[uOffs+1] = (uint8) fG;
			pPic[uOffs+2] = (uint8) fB;
			pPic[uOffs+3] = 0x7f;
		}
}

void GenText3(uint8 * pPic)
{
	for(int iY=0; iY<128; iY++)
		for(int iX=0; iX<128; iX++)
		{
			uint32 uOffs = (iY*128+iX)*4;

			float fR = 255;
			float fG = (((iX>>3)&1) ^ ((iY>>3)&1))*255;
			float fB = fG;

			pPic[uOffs+0] = (uint8) fR;
			pPic[uOffs+1] = (uint8) fG;
			pPic[uOffs+2] = (uint8) fB;
			pPic[uOffs+3] = 0x7f;
		}
}

void GenText4(uint8 * pPic)
{
	float fS = 255.0f / 63.5f;
	for(int iY=0; iY<128; iY++)
		for(int iX=0; iX<128; iX++)
		{
			uint32 uOffs = (iY*128+iX)*4;

			float fXpos = iX - 63.5f;
			float fYpos = iY - 63.5f;
			
			float fIntsSq = 63.5f*63.5f - (fXpos*fXpos + fYpos*fYpos);

			float fInts = (fIntsSq>0) ? sqrtf(fIntsSq) : 0;

			pPic[uOffs+0] = (uint8) 0;
			pPic[uOffs+1] = (uint8) (fS*fInts*0.635f);
			pPic[uOffs+2] = (uint8) (fS*fInts);
			pPic[uOffs+3] = (uint8) 0x7f;
		}
}

void SyncCNTStitch(CDmaGif * pDmaGif);

void AddTexToChain(CDmaGif * pDmaGif, uint32 uVRamPage, void * pImgData)
{
	// Add Texture to Chain
	SyncStitch(pDmaGif);
	pDmaGif->AttachSrcCntTag();

	// Set to packed mode (FLG)
	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(GIFTAG(4, 0, 0, 0, 0, 1));	// NLOOP, EOP, PRE, PRIM, FLG, NREG
	pDmaGif->add64(GIFTAG_REGS1(AD));

	// set transmission between buffers (dbp is wordaddr / 64)
	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(BITBLTBUF_SET(0,0,0,uVRamPage<<5,128>>6,0));	// SBP, SBW, SPSM, DBP, DBW, DPSM
	pDmaGif->add64(BITBLTBUF);

	// set transmission area between buffers
	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(TRXPOS_SET(0, 0, 0, 0, 0));	// SSAX, SSAY, DSAX, DSAY, DIR
	pDmaGif->add64(TRXPOS);

	// set size of transmission area
	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(TRXREG_SET(128, 128));	// RRW, RRh
	pDmaGif->add64(TRXREG);

	// set transmission direction
	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(TRXDIR_SET(0));	// XDIR
	pDmaGif->add64(TRXDIR);

	// Set to image mode (FLG)
	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(GIFTAG((128*128*4)>>4, 0, 0, 0, 2, 0));	// NLOOP, EOP, PRE, PRIM, FLG, NREG
	pDmaGif->add64(0);							// NoRegs
	pDmaGif->EndPacket();

	// Add Ref to texture data
	uint32 uBytesLeft = 128*128*4;
	uint32 uCurPtr = (uint32) pImgData;
	assert((uCurPtr&0xf)==0);
	while(uBytesLeft > 0)
	{
		uint32 uEndPtr = (uCurPtr&(~0xfff))+0x1000;
		uint32 uBSize = (uEndPtr - uCurPtr);
		assert((uBSize&0xf)==0);
		if(uBSize > uBytesLeft) uBSize = uBytesLeft;
		SyncStitch(pDmaGif);
		pDmaGif->AttachSrcRefTag((void *) uCurPtr, uBSize>>4);
		uCurPtr = uEndPtr;
		uBytesLeft -= uBSize;
	}

	// Close up
	SyncStitch(pDmaGif);
	pDmaGif->AttachSrcCntTag();

	uint32 uDstPtr = (uVRamPage*2048)/64;
	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(GIFTAG(2, 1, 0, 0, 0, 1));	// NLOOP, EOP, PRE, PRIM, FLG, NREG
	pDmaGif->add64(GIFTAG_REGS1(AD));

	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(0);
	pDmaGif->add64(TEXFLUSH);
	
	SyncCNTStitch(pDmaGif);
	pDmaGif->add64(TEX0_SET(uDstPtr, 2, 0, 7, 7, 1, 0, 0, 0, 0, 0, 0));
	pDmaGif->add64(TEX0_1);
	pDmaGif->EndPacket();
}

// Crappy way to do this
void SyncCNTStitch(CDmaGif * pDmaGif)
{
	uint32 uCur = (uint32) pDmaGif->GetCurLocation();
	assert((uCur&0xf)==0);
	if((uCur&0xff0)==0xff0)
	{
		pDmaGif->EndPacket();
		pDmaGif->AttachSrcNextTag((void *) (uCur+0x10));
		pDmaGif->EndPacket();
		pDmaGif->AttachSrcCntTag();
	}
}

